package de.szut.casino.lootboxes;

import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserRepository;
import de.szut.casino.user.UserService;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
public class LootBoxController {
    private final LootBoxRepository lootBoxRepository;
    private final UserService userService;
    private final LootBoxService lootBoxService;

    public LootBoxController(LootBoxRepository lootBoxRepository, UserRepository userRepository, UserService userService, LootBoxService lootBoxService) {
        this.lootBoxRepository = lootBoxRepository;
        this.userService = userService;
        this.lootBoxService = lootBoxService;
    }

    @GetMapping("/lootboxes")
    public List<LootBoxEntity> getAllLootBoxes() {
        return lootBoxRepository.findAll();
    }

    @PostMapping("/lootboxes/{id}")
    public ResponseEntity<Object> purchaseLootBox(@PathVariable Long id, @RequestHeader("Authorization") String token) {
        Optional<LootBoxEntity> optionalLootBox = lootBoxRepository.findById(id);
        if (optionalLootBox.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        LootBoxEntity lootBox = optionalLootBox.get();

        Optional<UserEntity> optionalUser = userService.getCurrentUser(token);
        if (optionalUser.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        UserEntity user = optionalUser.get();

        if (lootBoxService.hasSufficientBalance(user, lootBox.getPrice())) {
            Map<String, String> errorResponse = new HashMap<>();
            errorResponse.put("error", "Insufficient balance");
            return ResponseEntity.badRequest().body(errorResponse);
        }

        RewardEntity reward = lootBoxService.determineReward(lootBox);
        lootBoxService.handleBalance(user, lootBox, reward);

        return ResponseEntity.ok(reward);
    }
}
